﻿using System;

namespace NumberTypes
{
    /// <summary>
    /// Komplex szám típusa
    /// </summary>
    class Complex
    {
        /// <summary>
        /// Valós rész
        /// </summary>
        public Double Real { get; set; }

        /// <summary>
        /// Képzetes rész
        /// </summary>
        public Double Imaginary { get; set; }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        public Complex()
        {
            Real = 0;
            Imaginary = 0;
        }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        /// <param name="real">Valós rész</param>
        public Complex(Double real)
        {
            Real = real;
            Imaginary = 0;
        }

        /// <summary>
        /// Komplex szám létrehozása
        /// </summary>
        /// <param name="real">Valós rész</param>
        /// <param name="imaginary">Képzetes rész</param>
        public Complex(Double real, Double imaginary)
        {
            Real = real;
            Imaginary = imaginary;
        }

        /// <summary>
        /// Összeadás komplex számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Complex a, Complex b)
        {
            return new Complex(a.Real + b.Real, a.Imaginary + b.Imaginary);
        }

        /// <summary>
        /// Összeadás komplex és valós számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Complex a, Double b)
        {
            return new Complex(a.Real + b, a.Imaginary);
        }

        /// <summary>
        /// Összeadás valós és komplex számok között
        /// </summary>
        /// <param name="a">Első operandus</param>
        /// <param name="b">Második operandus</param>
        /// <returns>Összeadás eredménye</returns>
        public static Complex operator +(Double a, Complex b)
        {
            return new Complex(a + b.Real, b.Imaginary);
        }

        /// <summary>
        /// Valós számról komplex számra implicit konverzió.
        /// </summary>
        /// <param name="d">Való szám</param>
        /// <returns>Komplex szám</returns>
        public static implicit operator Complex(Double d)
        {
            return new Complex(d);
        }

        /// <summary>
        /// Racionális számról komplex számra explicit konverzió.
        /// </summary>
        /// <param name="r">Racionális szám</param>
        /// <returns>Komplex szám</returns>
        public static explicit operator Complex(Rational r)
        {
            return new Complex(r.Value);
        }

        /// <summary>
        /// Konjugálás
        /// </summary>
        public Complex Conjugation
        {
            get { return new Complex(Real, -Imaginary); }
        }

        /// <summary>
        /// Szöveggé alakítás
        /// </summary>
        /// <returns>String reprezentáció</returns>
        public override String ToString()
        {
            return String.Format("{0}{2}{1}i", Real, Imaginary, Imaginary >= 0 ? "+" : "-");
        }
    }
}
